#!/usr/bin/python
# -*- coding:utf-8 -*-

import types, re, string
import HTMLParser

class Node:
    """Abstract class  for DOM nodes, all instances must be constructed by
    Document.new_xxxxx() methods"""
    def __init__( self, doc ):
        self.__document = doc
        self.__parent   = None
        pass

    def get_document( self ):
        """Container document"""
        return doc
    
    def get_parent( self ):
        """Parental node"""
        return self.__parent

    def set_parent( self, parent ):
        """Set the parent node, this method calls Element.append_child method,
        you have to take care that this will become infinite loops when you
        override this method"""
        if isinstance( parent, Element ) == False:
            raise Exception( "Only DOM elements can be parents" )
        if self.__parent == parent: return
        if self.__parent != None and self.__parent != parent:
            self.__parent.remove_child( self )
            pass
        self.__parent = parent
        parent.append_child( self )
        pass

    def remove_parent( self, parent ):
        p = self.__parent
        self__parent = None
        if parent == p:
            parent.remove_child( self )
            pass
        pass
            

    def next_sibling( self, tag = None ):
        if self.__parent != None:
            return self.__parent.get_sibling( self, tag )
        return None

    pass

class Document ( Node ):
    """Lightweight DOM document"""
    def __init__( self, document_element = None ):
        Node.__init__( self, self )
        self.__element = document_element
        pass

    def set_document_element( self, elem ):
        self.__element = elem
        pass

    def new_element( self, tag, attrs = None ):
        """Instanciate new element, do not use constructor of the class from your code"""
        return Element( self, tag, attrs )

    def new_text( self, text ):
        """Instanciate new text node, do not use constructor of the class from your code"""
        return Text( self, text )

    def new_comment( self, comment ):
        """Instanciate new comment, do not use constructor of the class from your code"""
        return Comment( self, comment )

    def get_document_element( self ):
        return self.__element

    def get_element_at_path( self, path ):
        if self.__element == None:
            raise Exception( "no document element" )
        if path[ -1 ] == "/":
            path = path[ 0: -1 ]
            pass
        tag_root = path[ 1: ]
        next     = None
        if path[ 0 ] != "/":
            raise Exception( "path must begin with '/'" )
        else:
            path = path[ 1 : ]
            try:
                pos = path.index( "/" )
                tag_root = path[ 0: pos ]
                next = path[ pos + 1: ]
            except:
                pass
            pass
        if self.__element.get_tagname() == tag_root:
            if next == None:
                return self.__element
            else:
                return self.__element.get_element_at_path( next )
            pass
        return None

    def __repr__( self ):
        if self.__element == None: return ""
        return repr( self.__element )
    pass

class Element ( Node ):
    """Lightweight DOM element"""
    def __init__( self, doc, tag, attrs = None ):
        Node.__init__( self, doc )
        self.__children = []
        self.__tagname = tag
        if type( attrs ) == types.ListType:
            self.__attribute = {}
            for attr in attrs:
                self.__attribute[ attr[ 0 ] ] = attr[ 1 ]
                pass
            pass
        elif type( attrs ) == types.DictionaryType:
            self.__attribute = attrs
            pass
        elif attrs == None:
            self.__attribute = {}
            pass
        pass

    def get_text( self, separator = "" ):
        """Get whole text included in the element"""
        text = None
        for child in self.__children:
            if isinstance( child, Text ):
                if text != None: text += separator
                else: text = ""
                text += child.get_text()
                pass
            elif isinstance( child, Element ):
                if text != None: text += separator
                else: text = ""
                text += child.get_text( separator )
                pass
            pass
        if text == None: return ""
        return text

    def has_attribute( self, label ):
        for key in self.__attribute.keys():
            if key == label: return True
            pass
        return False

    def get_attribute( self, label ):
        return self.__attribute[ label ]

    def __contains_element( self ):
        for child in self.__children:
            if isinstance( child, Element ): return True
            pass
        return False

    def get_sibling( self, node, tag = None ):
        """enumerate children"""
        start = -1
        for i in range( 0, len( self.__children ) ):
            if node is self.__children[ i ]:
                start = i
                break
            pass
        if start < 0:
            raise Exception( "not child of the element" )
        pos = start + 1
        if tag == None: # without tag filtration
            if pos >= len( self.__children ):
                return None
            return self.__children[ pos ]

        if tag == "": tag = node.get_tagname()
    
        if isinstance( node, Element ):
            #tag = node.get_tagname()
            for i in range( pos, len( self.__children ) ):
                if isinstance( self.__children[ i ], Element ):
                    if self.__children[ i ].get_tagname() == tag:
                        return self.__children[ i ]
                    pass
                pass
            pass
        elif isinstance( node, Text ):
            for i in range( pos, len( self.__children ) ):
                if isinstance( self.__children[ i ], Text ):
                    return self.__chilren[ i ]
                pass
            pass
        raise Exception( "no more sibs" )
        
    def get_element_at_path( self, path ):
        if path[ 0 ] == '/':
            return self.get_document().get_element_at_path( path )
        #print "required path ", path, self.__tagname
        name = path
        next = None
        try:
            pos  = path.index( '/' )
            name = path[ 0 : pos ]
            while path[ pos ] == '/': pos += 1
            next = path[ pos: ]
        except:
            pass

        #print "NAME  ==> ", name, self.get_tagname()
        m = re.match( "(\\w+),(\\d+)", name )
        count_sib = 1
        if m:
            name = m.group( 1 )
            count_sib = string.atoi( m.group( 2 ) )
            pass
        elem = None
        for child in self.__children:
            if isinstance( child, Element ) == False: continue
            if child.get_tagname() == name:
                count_sib -= 1
                if count_sib > 0: continue
                if next == None:
                    elem = child
                    break
                else:
                    elem = child.get_element_at_path( next )
                    if elem != None: break
                    pass
                pass
            pass
        return elem

    def get_first_child( self ):
        if len( self.__children ) > 0: return self.__children[ 0 ]
        return None

    def remove_child( self, node ):
        if not node.get_parent() is self: return
        for i in range( len( self.__children ) ):
            if self.__children[ i ] is node:
                self.__children = self.__children[ 0: i ] + self.__children[ i + 1: ]
                node.remove_parent( node )
                pass
            pass
        pass

    def get_children( self ):
        """Retrieve all descendant nodes"""
        return self.__children
    
    def append_child( self, node ):
        if node == None or node is self: return
        if isinstance( node, Node ):
            for child in self.__children:
                if child is node: return
                pass
            self.__children.append( node )
            node.set_parent( self )
            pass
        else:
            raise TypeError( "%s is not Node object" % repr( node ) )
        pass

    def get_tagname( self ):
        return self.__tagname

    def __count_depth( self ):
        if self.get_parent() == None: return 0
        return self.get_parent().__count_depth() + 1

    def __get_absolute_path( self ):
        if self.get_parent() == None: return "/" + self.__tagname
        return self.get_parent().__get_absolute_path() + "/" + self.__tagname

    def get_string_with_indent( self, spaces = 0 ):
        indent = ""
        for i in range( spaces ): indent += " "
        line = "%s<%s" % ( indent, self.__tagname )

        # add attribute
        for key in self.__attribute.keys():
            value = self.__attribute[ key ]
            if value == None:
                line += " " + key
            else:
                line += " %s=\"%s\"" % ( key, value )
                pass
            pass

        if len( self.__children ) == 0:
            line += "/>\n"
        else:
            #line += ">\n"
            line += "> <!-- %s-->\n" % ( self.__get_absolute_path() )
            for child in self.__children:
                if isinstance( child, Element ):
                    line += child.get_string_with_indent( spaces + 1 )
                else:
                    line += repr( child )
                    pass
                pass
            line += "%s</%s>\n" % (indent, self.__tagname )
            pass
        return line
    
    def __repr__( self ):
        return self.get_string_with_indent( 0 )
    pass

class Text( Node ):
    def __init__( self, doc, text ):
        Node.__init__( self, doc )
        self.__text = text
        pass

    def get_text( self ):
        return self.__text

    def __repr__( self ):
        return self.__text + "\n"
    pass

class Comment( Node ):
    def __init__( self, doc, text ):
        Node.__init__( self, doc )
        self.__text = text
        pass

    def get_text( self ):
        return self.__text

    def __repr__( self ):
        return "<!--%s-->\n" % self.__text
    pass


class DOMConverter ( HTMLParser.HTMLParser ):
    __IGNORE_LIST = [ 'meta', 'br' ]

    def __init__( self ):
        HTMLParser.HTMLParser.__init__( self )
        self.__purging = self.__IGNORE_LIST
        self.__doc  = Document()
        self.__elem = None
        pass

    def add_ignoring_tag( self, tag ):
        if self.__purging.count( tag ) == 0:
            self.__purging.append( tag )
            pass
        return

    def remove_ignoring_tag( self, tag ):
        try:
            n = self.___purging.index( tag )
            self.__purging = self.__purging[ 0: n ] + self.__purging[ n + 1: ]
        except:
            pass
        return

    def handle_data( self, data ):
        data = data.strip( " \t\n\r" )
        if len( data ) == 0 or self.__elem == None: return
        self.__elem.append_child( self.__doc.new_text( data ) )
        pass

    def __can_ignore( self, tag ):
        for t in self.__IGNORE_LIST:
            if t == tag: return True
            pass
        return False

    def handle_starttag( self, tag, attrs ):
        tag = tag.lower()

        if self.__can_ignore( tag ):return
        #print "START TAG ", tag

        # top element
        if self.__elem == None:
            self.__elem = self.__doc.new_element( tag, attrs )
            self.__doc.set_document_element( self.__elem )
            return

        # compensate
        tag_parent = self.__elem.get_tagname()
        if tag == 'td' or tag == 'th':
            if tag_parent == 'td' or tag_parent == 'th':
                self.handle_endtag( tag_parent )
                pass
            pass
        elif tag == 'tr':
            if tag_parent == 'td' or tag_parent == 'th':
                self.handle_endtag(  tag_parent )
                pass
            tag_parent = self.__elem.get_tagname()
            if tag_parent == 'tr':
                self.handle_endtag( tag )
                pass
            pass

        # create new element
        elem = self.__doc.new_element( tag, attrs )
        self.__elem.append_child( elem )
        self.__elem = elem

        # close tag immediately
        if tag == 'img' or tag == 'hr' or tag == 'link' or tag == 'area' or tag == 'input':
            self.__elem = self.__elem.get_parent()
            pass
        return

    def handle_endtag( self, tag ):
        tag = tag.lower()
        #images
        #if tag == 'img': return

        # unbalanced tag
        if self.__elem == None or self.__elem.get_tagname() != tag:
            return
        
        if self.__can_ignore( tag ):return

        self.__elem = self.__elem.get_parent()
        #print "NEXT PARENT => ", self.elem.get_tagname()
        pass

    def handle_comment( self, comment ):
        if self.__elem:
            self.__elem.append_child( self.__doc.new_comment( comment ) )
            pass
        pass

    def get_document( self ):
        return self.__doc
    pass

if __name__ == "__main__":
    import sys
    filename = sys.argv[ 1 ]
    data = ""
    p = DOMConverter()
    for line in open( filename ).readlines():
        line = line.strip()
        #if line.count( "'</SCR'" ) > 0: continue
        #if re.search( "'<SCR'", line, re.I ): continue
        #line = unicode( line.strip(), 'iso-2022-jp' ).encode( 'utf-8' )
        #print line
        p.feed( line )
        pass
    print p.get_document()
    sys.exit()
                
    for line in open( filename ).readlines():
        try:
            line = unicode( line.strip(), 'euc-jp' ).encode( 'utf-8' )
        except:
            print line
            continue
        try:
            line = re.sub( "alt=[^\"]([^\\\"\\s>]+)", "alt=\"\\1\"", line )
            p.feed( line )
        except:
            print line
            break
            continue
        pass
    doc = p.get_document()
    #print doc
    #elem = doc.get_element_at_path( "/html/body/table/tr/td/div/tab" )#.next_sibling().next_sibling().next_sibling()
    print doc
    #output = repr( p.get_document() )
    #print repr( p.get_document() )#.encode( 'utf-8' )
    sys.exit()

